package org.amse.vbut.vzab;

/**
 * VZabColor class
 * 
 * @author But Vasiliy
 *
 */
public class VZabColor {
	
    public final static VZabColor WHITE      = new VZabColor(255, 255, 255);
    public final static VZabColor LIGHT_GRAY = new VZabColor(192, 192, 192);
    public final static VZabColor GRAY       = new VZabColor(128, 128, 128);
    public final static VZabColor DARK_GRAY  = new VZabColor(64, 64, 64);
    public final static VZabColor BLACK 	 = new VZabColor(0, 0, 0);
    public final static VZabColor RED        = new VZabColor(255, 0, 0);
    public final static VZabColor PINK       = new VZabColor(255, 175, 175);
    public final static VZabColor ORANGE 	 = new VZabColor(255, 200, 0);
    public final static VZabColor YELLOW 	 = new VZabColor(255, 255, 0);
    public final static VZabColor GREEN 	 = new VZabColor(0, 255, 0);
    public final static VZabColor MAGENTA	 = new VZabColor(255, 0, 255);
    public final static VZabColor CYAN       = new VZabColor(0, 255, 255);
    public final static VZabColor BLUE       = new VZabColor(0, 0, 255);

	private final int myRGB;
	
	/**
	 * Constructs VZabColor object with specified
	 * red, green and blue color components.
	 */
	public VZabColor(int r, int g, int b) {
		myRGB = (r & 0xff) << 16 | ((g & 0xff) << 8) | (b & 0xff);
	}
	
	/**
	 * Constructs VZabColor object with specified
	 * rgb value. rgb value format: 0xrrggbb.
	 */
	public VZabColor(int rgb) {
		myRGB = rgb & 0x00ffffff;
	}
	
	/**
	 * Constructs VZabColor object with rgb value 
	 * stored in string as integer in radix radix.<br>
	 * <br>
	 * This constructor is equivalent to call:<br>
	 * new VZabColor( Integer.parseInt(color, radix) );<br>
	 */
	public VZabColor(String color, int radix) {
		myRGB = Integer.parseInt(color, radix) & 0x00ffffff;
	}
	
	/**
	 * Returns blue component of this color.
	 * @return blue component of this color.
	 */
	public int getBlue() {
		return myRGB & 0xff;
	}

	/**
	 * Returns green component of this color.
	 * @return green component of this color.
	 */
	public int getGreen() {
		return (myRGB >>> 8) & 0xff;
	}

	/**
	 * Returns red component of this color.
	 * @return red component of this color.
	 */
	public int getRed() {
		return (myRGB >>> 16) & 0xff;
	}
	
	public boolean equals(Object o) {
		if (o == this) {
			return true;
		}
		
		if (!(o instanceof VZabColor)) {
			return false;
		}
		
		VZabColor c = (VZabColor)o;
		
		return c.getRed() == this.getRed()
			&& c.getGreen() == this.getGreen()
			&& c.getBlue() == this.getBlue();
	}
	
	public String toString() {
		return "[r=" + getRed() + ",g=" + getGreen() + ",b=" + getBlue() + "]";
	}
	
	/**
	 * Returns string representation of this color 
	 * in hexadecimal format.
	 * @return string representation of this color.
	 */
	public String toHexString() {
		return String.format("%1$02x%2$02x%3$02x", 
				Integer.valueOf(getRed()), 
				Integer.valueOf(getGreen()), 
				Integer.valueOf(getBlue()));
	}
	
}
