package org.amse.vbut.vzab.io;

import org.amse.vbut.vzab.model.*;

/**
 * Interface of the reader
 * 
 * @author But Vasiliy
 *
 */
public interface IReader {
	
	/**
	 * Reads the game from specified file and creates new Game 
	 * object with the loaded game. 
	 * @param filename - name of file to read the game from
	 * @param minwidth - minimum width of the board
	 * @param minheight - minimum height of the board
	 * @param maxwidth - maximum width of the board
	 * @param maxheight - maximum height of the board
	 * @return Game object
	 */
	IGame readGame(String filename, int minwidth, int minheight, int maxwidth, int maxheight);
	
	/**
	 * Reads player list from file.
	 * @param filename - name of file to read from.
	 * @return list of players.
	 */
	java.util.List<IPlayer> readPlayers(String filename);
	
	/**
	 * Reads initializing settings from file filename.
	 * @param filename - name of file to read settings from
	 * @return initializing settings have been read from specified file.
	 */
	ISettings readSettings(String filename);
	
	/**
	 * Returns ISettings object with deault settings values.
	 * @return ISettings object with deault settings values.
	 */
	ISettings getDefaultSettings();
	
	/**
	 * Reads readme information from specified file.
	 * @param filename - name of file to read from
	 * @return string contains readme information.
	 */
	String readReadme(String filename);
}
