package org.amse.vbut.vzab.model.impl;

import java.awt.*;
import javax.swing.*;

import org.amse.vbut.vzab.model.IPlayer;

/**
 * Player class
 * 
 * @author But Vasiliy
 *
 */
class Player implements IPlayer {

	private String myName;
	private boolean myIsComputer;
	
	private ImageIcon myImageAlive;
	private ImageIcon myImageKilled;
	private ImageIcon myImageBase;
	private ImageIcon myImageBaseKilled;
	private ImageIcon myImageSmall;
	
	private String myPath;

	/**
	 * Creates player with specified name, isComputer flag,
	 * imageAlive, imageKilled, imageBase, imageBaseKilled images and
	 * path value.
	 */
	public Player(String name, boolean isComputer, ImageIcon imageAlive,
			ImageIcon imageKilled, ImageIcon imageBase, ImageIcon imageBaseKilled, 
			String path) {
		myName = name;
		myIsComputer = isComputer;
		
		myImageAlive = imageAlive;
		myImageKilled = imageKilled;
		myImageBase = imageBase;
		myImageBaseKilled = imageBaseKilled;
		
		if (myImageAlive == null) {
			myImageSmall = null;
		} else {
			myImageSmall = new ImageIcon( myImageAlive.getImage().getScaledInstance(-1, 
					IPlayer.SMALL_IMAGE_SIZE, Image.SCALE_DEFAULT) );
		}
		
		myPath = path;
	}
	
	/**
	 * Creates new copy of the specified player
	 */
	public Player(IPlayer player) {
		myName = player.getName();
		myIsComputer = player.isComputer();

		myImageAlive = player.getImageAlive();
		myImageKilled = player.getImageKilled();
		myImageBase = player.getImageBase();
		myImageBaseKilled = player.getImageBaseKilled();
		myImageSmall = player.getImageSmall();
		
		myPath = player.getPath();
	}
	
	public final String getName() {
		return myName;
	}
	
	public final void setName(String name) {
		myName = name;
	}
	
	public String toString() {
		return "[" + getName() + "," + (isComputer() ? "computer" : "human") + "," + getPath() + "]";
	}
	
	public boolean equals(Object o) {
		if (o == this) {
			return true;
		}
		
		if (o == null || !(o instanceof IPlayer)) {
			return false;
		}
		
		IPlayer p = (IPlayer)o;
		
		return getPath().equals(p.getPath());
	}

	public final boolean isComputer() {
		return myIsComputer;
	}

	public final void setComputer(boolean b) {
		myIsComputer = b;
	}

	public ImageIcon getImageAlive() {
		return myImageAlive;
	}

	public ImageIcon getImageBase() {
		return myImageBase;
	}

	public ImageIcon getImageBaseKilled() {
		return myImageBaseKilled;
	}

	public ImageIcon getImageKilled() {
		return myImageKilled;
	}

	public void setImageAlive(ImageIcon image) {
		myImageAlive = image;
	}

	public void setImageBase(ImageIcon image) {
		myImageBase = image;
	}

	public void setImageBaseKilled(ImageIcon image) {
		myImageBaseKilled = image;
	}

	public void setImageKilled(ImageIcon image) {
		myImageKilled = image;
	}

	public String getPath() {
		return myPath;
	}

	public void setPath(String filename) {
		myPath = filename;
	}

	public ImageIcon getImageSmall() {
		return myImageSmall;
	}

	public void setImageSmall(ImageIcon image) {
		myImageSmall = image;
	}

}
