package org.amse.vbut.vzab.tests;

import org.amse.vbut.vzab.model.*;
import org.amse.vbut.vzab.model.impl.*;
import java.util.*;

import junit.framework.TestCase;

public class BoardTests extends TestCase {

	private IBoard myBoard;
	
	private static IBoard newBoard(int w, int h) {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0"));
		pls.add(1, ModelFactory.newPlayer("pl1"));
		
		IGame g = ModelFactory.newGame(w, h, pls);

		return g.getBoard();
	}
	
	public void test01() {
		myBoard = newBoard(10, 12);
		
		assertEquals("Wrong getWidth() value", 10, myBoard.getWidth());
		assertEquals("Wrong getHeight() value", 12, myBoard.getHeight());
	}
	
	public void test02() {
		myBoard = newBoard(15, 10);
		
		assertSame(null, myBoard.getPiece(5, 3));
		
		IPlayer pl = ModelFactory.newPlayer("Vasiliy"); 
		
		myBoard.setPiece(5, 3, pl);
		
		assertEquals("[player=[Vasiliy,human,],alive=true]", 
				myBoard.getPiece(5, 3).toString());
		
		myBoard.getPiece(5, 3).kill();
		
		assertEquals("[player=[Vasiliy,human,],alive=false]", 
				myBoard.getPiece(5, 3).toString());
		
		pl.setName("ggg");
		pl.setComputer(true);
		pl.setPath("/players/blue-player");

		assertEquals("[player=[ggg,computer,/players/blue-player],alive=false]", 
				myBoard.getPiece(5, 3).toString());
	}
	
	public void test03() {
		myBoard = newBoard(15, 10);
		
		assertSame(null, myBoard.getPiece(5, -1));
		assertSame(null, myBoard.getPiece(-1, 3));
	}
	
	public void test04() {
		final String ctrlValue = "[width=10,height=25]";
		
		myBoard = newBoard(10, 25);
		
		myBoard.clear();

		String str = myBoard.toString();

		assertEquals(ctrlValue, str);
	}
	
	public void test05() {
		final String ctrlValue = "[width=10,height=15\n"
			+ "[x=0,y=0,[player=[0,computer,/players/blue-player],alive=true]]\n"
			+ "[x=2,y=1,[player=[0,computer,/players/blue-player],alive=true]]\n"
			+ "[x=9,y=14,[player=[1,human,/players/red-player],alive=true]]]";

		myBoard = newBoard(10, 15);
		
		IPlayer player0 = ModelFactory.newPlayer("0", true, "/players/blue-player");
		IPlayer player1 = ModelFactory.newPlayer("1", false, "/players/red-player");
	
		myBoard.setPiece(0, 0,  player0);
		myBoard.setPiece(9, 14, player1);
		myBoard.setPiece(2, 1,  player0);
	
		String str = myBoard.toString();
	
		assertEquals(ctrlValue, str);
	}	

	public void test06() {
		final String ctrlValue = "[width=10,height=15\n"
			+ "[x=0,y=0,[player=[0,computer,/players/blue-player],alive=true]]\n"
			+ "[x=2,y=1,[player=[0,computer,/players/blue-player],alive=false]]\n"
			+ "[x=2,y=2,[player=[1,human,/players/red-player],alive=true]]\n"
			+ "[x=9,y=14,[player=[1,human,/players/red-player],alive=true]]]";

		myBoard = newBoard(10, 15);

		IPlayer player0 = ModelFactory.newPlayer("0", true, "/players/blue-player");
		IPlayer player1 = ModelFactory.newPlayer("1", false, "/players/red-player");

		myBoard.setPiece(0, 0,  player0);
		myBoard.setPiece(9, 14, player1);
		myBoard.setPiece(2, 1,  player0);
		myBoard.setPiece(2, 2,  player1);

		myBoard.getPiece(2, 1).kill();

		String str = myBoard.toString();

		assertEquals(ctrlValue, str);
	}
	
	public void test07() {
		final String ctrlValue = "[width=10,height=15]";
		
		myBoard = newBoard(10, 15);
		
		IPlayer player0 = ModelFactory.newPlayer("0");
		IPlayer player1 = ModelFactory.newPlayer("1");

		myBoard.setPiece(0, 0, player0);
		myBoard.setPiece(9, 14, player1);
		myBoard.setPiece(2, 1, player0);
		myBoard.setPiece(2, 2, player1);

		myBoard.getPiece(2, 1).kill();
		
		myBoard.clear();

		String str = myBoard.toString();

		assertEquals(ctrlValue, str);
	}
	
	public void test08() {
		final String ctrlValue = "[width=10,height=15\n"
			+ "[x=0,y=0,[player=[0,computer,/players/blue-player],alive=true]]\n"
			+ "[x=2,y=1,[player=[0,computer,/players/blue-player],alive=false]]\n"
			+ "[x=2,y=2,[player=[1,human,/players/red-player],alive=true]]\n"
			+ "[x=9,y=14,[player=[1,human,/players/red-player],alive=true]]]";

		myBoard = newBoard(10, 15);

		IPlayer player0 = ModelFactory.newPlayer("0", true, "/players/blue-player");
		IPlayer player1 = ModelFactory.newPlayer("1", false, "/players/red-player");

		myBoard.setPiece(0, 0, player0);
		myBoard.setPiece(9, 14, player1);
		myBoard.setPiece(2, 1, player0);
		myBoard.setPiece(5, 10, player1);
		myBoard.setPiece(2, 2, player1);

		myBoard.getPiece(2, 1).kill();
		
		myBoard.clear();

		myBoard.setPiece(0, 0, player0);
		myBoard.setPiece(9, 14, player1);
		myBoard.setPiece(2, 1, player0);
		myBoard.setPiece(2, 2, player1);

		myBoard.getPiece(2, 1).kill();

		String str = myBoard.toString();

		assertEquals(ctrlValue, str);
	}

}
