package org.amse.vbut.vzab.tests;

import org.amse.vbut.vzab.*;
import org.amse.vbut.vzab.model.*;
import org.amse.vbut.vzab.model.impl.*;
import java.util.*;

import junit.framework.TestCase;

public class GameTests extends TestCase {

	private IGame myGame;
	
	public void test01() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0"));
		pls.add(1, ModelFactory.newPlayer("pl1"));
		pls.add(2, ModelFactory.newPlayer("pl2"));
		pls.add(3, ModelFactory.newPlayer("pl3"));
		
		myGame = ModelFactory.newGame(10, 15, pls);
		
		assertEquals(4, myGame.getPlayers().size());
	}

	public void test02() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0"));
		pls.add(1, ModelFactory.newPlayer("pl1"));
		
		myGame = ModelFactory.newGame(10, 15, pls);

		assertEquals(2, myGame.getPlayers().size());
		assertEquals(0, myGame.getCurrentPlayer());
		
		assertEquals("[player=[pl0,human,],alive=true]",
				myGame.getBoard().getPiece(0, 0).toString());
		assertEquals("[player=[pl1,human,],alive=true]",
				myGame.getBoard().getPiece(9, 14).toString());
	}
	
	public void test03() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0", false, "1"));
		pls.add(1, ModelFactory.newPlayer("pl1", false, "2"));
		pls.add(2, ModelFactory.newPlayer("pl2", false, "3"));
		pls.add(3, ModelFactory.newPlayer("pl3", false, "4"));
		
		myGame = ModelFactory.newGame(10, 15, pls);
		
		assertEquals(10, myGame.getBoard().getWidth());
		assertEquals(15, myGame.getBoard().getHeight());
		assertEquals(4, myGame.getPlayers().size());
		assertEquals(0, myGame.getCurrentPlayer());
		
		assertEquals("[player=[pl0,human,1],alive=true]",
				myGame.getBoard().getPiece(0, 0).toString());
		assertEquals("[player=[pl1,human,2],alive=true]",
				myGame.getBoard().getPiece(9, 14).toString());
		assertEquals("[player=[pl2,human,3],alive=true]",
				myGame.getBoard().getPiece(0, 14).toString());
		assertEquals("[player=[pl3,human,4],alive=true]",
				myGame.getBoard().getPiece(9, 0).toString());
	}
	
	public void test04() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0"));
		
		try {
			myGame = ModelFactory.newGame(10, 15, pls);
		} catch(VZabException e) {
			return;
		}
		
		fail();
	}

	public void test05() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0"));
		pls.add(1, ModelFactory.newPlayer("pl1"));
		
		try {
			myGame = ModelFactory.newGame(1, 15, pls);
		} catch(VZabException e) {
			return;
		}
		
		fail();
	}
	
	public void test06() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0"));
		pls.add(1, ModelFactory.newPlayer("pl1"));
		
		myGame = ModelFactory.newGame(10, 10, pls);

		assertFalse(myGame.canDoMove(7, 8, myGame.getBoard().getPiece(9, 9), true));
		assertTrue(myGame.canDoMove(2, 1, myGame.getBoard().getPiece(0, 0), true));
		assertTrue(myGame.canDoMove(1, 2, myGame.getBoard().getPiece(0, 0), true));
		assertFalse(myGame.canDoMove(2, 1, null, true));
		
		myGame.doMove(2, 1, myGame.getBoard().getPiece(0, 0));
		
		assertEquals(10, myGame.getBoard().getWidth());
		assertEquals(10, myGame.getBoard().getHeight());
		assertEquals(2, myGame.getPlayers().size());
		assertEquals(1, myGame.getCurrentPlayer());
		
		assertTrue(myGame.canDoMove(7, 8, myGame.getBoard().getPiece(9, 9), true));
		assertFalse(myGame.canDoMove(2, 1, myGame.getBoard().getPiece(0, 0), true));
		assertFalse(myGame.canDoMove(2, 1, null, true));

		myGame.doMove(7, 8, myGame.getBoard().getPiece(9, 9));
		
		assertFalse(myGame.canDoMove(8, 1, myGame.getBoard().getPiece(9, 9), true));
		assertEquals(0, myGame.getCurrentPlayer());
		assertFalse(myGame.canDoMove(2, 1, myGame.getBoard().getPiece(0, 0), true));
		assertTrue(myGame.canDoMove(1, 2, myGame.getBoard().getPiece(0, 0), true));
		
		myGame.doMove(1, 2, myGame.getBoard().getPiece(0, 0));

		assertEquals(1, myGame.getCurrentPlayer());
	}
	
	public void test07() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0", false, "1"));
		pls.add(1, ModelFactory.newPlayer("pl1", false, "2"));
		pls.add(2, ModelFactory.newPlayer("pl2", false, "3"));
		pls.add(3, ModelFactory.newPlayer("pl3", false, "4"));
		
		myGame = ModelFactory.newGame(8, 8, pls);

		myGame.doMove(2, 1, myGame.getBoard().getPiece(0, 0));
		myGame.doMove(5, 6, myGame.getBoard().getPiece(7, 7));
		myGame.doMove(1, 5, myGame.getBoard().getPiece(0, 7));
		myGame.doMove(5, 1, myGame.getBoard().getPiece(7, 0));
		
		myGame.doMove(3, 3, myGame.getBoard().getPiece(2, 1));
		myGame.doMove(3, 5, myGame.getBoard().getPiece(5, 6));
		myGame.doMove(3, 4, myGame.getBoard().getPiece(1, 5));
		myGame.doMove(4, 3, myGame.getBoard().getPiece(5, 1));


		assertTrue(myGame.getBoard().getPiece(5, 1).isAlive());
		assertTrue(myGame.getBoard().getPiece(4, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(2, 1).isAlive());
		assertFalse(myGame.getBoard().getPiece(3, 3).isAlive());
		assertFalse(myGame.getBoard().getPiece(3, 4).isAlive());
		assertFalse(myGame.getBoard().getPiece(3, 5).isAlive());
		
		assertFalse(myGame.canDoMove(1, 4, myGame.getBoard().getPiece(2, 1), true));
		assertFalse(myGame.canDoMove(3, 3, myGame.getBoard().getPiece(2, 1), true));
		assertFalse(myGame.canDoMove(5, 5, myGame.getBoard().getPiece(2, 1), true));
		assertFalse(myGame.canDoMove(2, 3, myGame.getBoard().getPiece(2, 1), true));
		
		myGame.doMove(4, 2, myGame.getBoard().getPiece(2, 1));
		
		assertFalse(myGame.getBoard().getPiece(5, 1).isAlive());
		assertFalse(myGame.getBoard().getPiece(4, 3).isAlive());
	}
	
	public void test08() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0", false, "1"));
		pls.add(1, ModelFactory.newPlayer("pl1", false, "2"));
		pls.add(2, ModelFactory.newPlayer("pl2", false, "3"));
		pls.add(3, ModelFactory.newPlayer("pl3", false, "4"));
		
		myGame = ModelFactory.newGame(8, 8, pls);

		myGame.doMove(2, 1, myGame.getBoard().getPiece(0, 0));
		myGame.doMove(5, 6, myGame.getBoard().getPiece(7, 7));
		myGame.doMove(1, 5, myGame.getBoard().getPiece(0, 7));
		myGame.doMove(5, 1, myGame.getBoard().getPiece(7, 0));
		
		myGame.doMove(3, 3, myGame.getBoard().getPiece(2, 1));
		myGame.doMove(3, 5, myGame.getBoard().getPiece(5, 6));
		myGame.doMove(3, 4, myGame.getBoard().getPiece(1, 5));
		myGame.doMove(4, 3, myGame.getBoard().getPiece(5, 1));

		myGame.doMove(4, 2, myGame.getBoard().getPiece(2, 1));
		myGame.doMove(6, 4, myGame.getBoard().getPiece(5, 6));
		myGame.doMove(2, 3, myGame.getBoard().getPiece(1, 5));
		myGame.doMove(6, 2, myGame.getBoard().getPiece(7, 0));

		myGame.doMove(1, 3, myGame.getBoard().getPiece(2, 1));
		myGame.doMove(5, 2, myGame.getBoard().getPiece(6, 4));
		
		
		assertEquals(2, myGame.getCurrentPlayer());
		
		assertFalse(myGame.getBoard().getPiece(3, 3).isAlive());
		assertFalse(myGame.getBoard().getPiece(2, 3).isAlive());
		assertFalse(myGame.getBoard().getPiece(4, 3).isAlive());
		assertFalse(myGame.getBoard().getPiece(4, 2).isAlive());
		assertFalse(myGame.getBoard().getPiece(5, 1).isAlive());
		assertFalse(myGame.getBoard().getPiece(6, 2).isAlive());
		assertTrue(myGame.getBoard().getPiece(5, 2).isAlive());
		assertTrue(myGame.getBoard().getPiece(1, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(6, 4).isAlive());
		
		myGame.undoMove(); // (5, 2)

		assertEquals(1, myGame.getCurrentPlayer());
		
		assertFalse(myGame.getBoard().getPiece(3, 3).isAlive());
		assertFalse(myGame.getBoard().getPiece(2, 3).isAlive());
		assertFalse(myGame.getBoard().getPiece(4, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(4, 2).isAlive());
		assertFalse(myGame.getBoard().getPiece(5, 1).isAlive());
		assertTrue(myGame.getBoard().getPiece(6, 2).isAlive());
		assertTrue(myGame.getBoard().getPiece(1, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(6, 4).isAlive());
		
		assertSame(null, myGame.getBoard().getPiece(5, 2));

		myGame.undoMove(); // (1, 3)

		assertEquals(0, myGame.getCurrentPlayer());

		assertFalse(myGame.getBoard().getPiece(3, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(2, 3).isAlive());
		assertFalse(myGame.getBoard().getPiece(4, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(4, 2).isAlive());
		assertFalse(myGame.getBoard().getPiece(5, 1).isAlive());
		assertTrue(myGame.getBoard().getPiece(6, 2).isAlive());
		assertTrue(myGame.getBoard().getPiece(6, 4).isAlive());

		assertSame(null, myGame.getBoard().getPiece(1, 3));
		assertSame(null, myGame.getBoard().getPiece(5, 2));

		myGame.undoMove(); // (6, 2)

		assertEquals(3, myGame.getCurrentPlayer());

		assertFalse(myGame.getBoard().getPiece(3, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(2, 3).isAlive());
		assertFalse(myGame.getBoard().getPiece(4, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(4, 2).isAlive());
		assertFalse(myGame.getBoard().getPiece(5, 1).isAlive());
		assertTrue(myGame.getBoard().getPiece(6, 4).isAlive());

		assertSame(null, myGame.getBoard().getPiece(6, 2));
		assertSame(null, myGame.getBoard().getPiece(1, 3));
		assertSame(null, myGame.getBoard().getPiece(5, 2));

		myGame.undoMove(); // (2, 3)

		assertEquals(2, myGame.getCurrentPlayer());

		assertFalse(myGame.getBoard().getPiece(3, 3).isAlive());
		assertFalse(myGame.getBoard().getPiece(4, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(4, 2).isAlive());
		assertFalse(myGame.getBoard().getPiece(5, 1).isAlive());
		assertTrue(myGame.getBoard().getPiece(6, 4).isAlive());

		assertSame(null, myGame.getBoard().getPiece(2, 3));
		assertSame(null, myGame.getBoard().getPiece(6, 2));
		assertSame(null, myGame.getBoard().getPiece(1, 3));
		assertSame(null, myGame.getBoard().getPiece(5, 2));

		myGame.undoMove(); // (6, 4)

		assertEquals(1, myGame.getCurrentPlayer());

		assertFalse(myGame.getBoard().getPiece(3, 3).isAlive());
		assertFalse(myGame.getBoard().getPiece(4, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(4, 2).isAlive());
		assertFalse(myGame.getBoard().getPiece(5, 1).isAlive());

		assertSame(null, myGame.getBoard().getPiece(6, 4));
		assertSame(null, myGame.getBoard().getPiece(2, 3));
		assertSame(null, myGame.getBoard().getPiece(6, 2));
		assertSame(null, myGame.getBoard().getPiece(1, 3));
		assertSame(null, myGame.getBoard().getPiece(5, 2));
	
		myGame.undoMove(); // (4, 2)

		assertEquals(0, myGame.getCurrentPlayer());

		assertFalse(myGame.getBoard().getPiece(3, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(4, 3).isAlive());
		assertTrue(myGame.getBoard().getPiece(5, 1).isAlive());

		assertSame(null, myGame.getBoard().getPiece(4, 2));
		assertSame(null, myGame.getBoard().getPiece(6, 4));
		assertSame(null, myGame.getBoard().getPiece(2, 3));
		assertSame(null, myGame.getBoard().getPiece(6, 2));
		assertSame(null, myGame.getBoard().getPiece(1, 3));
		assertSame(null, myGame.getBoard().getPiece(5, 2));

		myGame.undoMove();
		myGame.undoMove();
		myGame.undoMove();
		myGame.undoMove();
		myGame.undoMove();
		myGame.undoMove();
		myGame.undoMove();
		assertTrue(myGame.canUndoMove());
		myGame.undoMove();

		assertEquals(0, myGame.getCurrentPlayer());
		
		assertFalse(myGame.canUndoMove());
		/*try {
			myGame.undoMove();
		} catch(VZabException e) {
			return;
		}
		
		fail("Undo must throw exception.");*/
	}

	public void test09() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0"));
		pls.add(1, ModelFactory.newPlayer("pl1"));
		
		myGame = ModelFactory.newGame(10, 10, pls);

		assertEquals(2, myGame.getPlayers().size());
		assertEquals(0, myGame.getCurrentPlayer());

		myGame.setCurrentPlayer(1);
		assertEquals(1, myGame.getCurrentPlayer());

		myGame.setCurrentPlayer(0);
		assertEquals(0, myGame.getCurrentPlayer());
	}
	
	public void test10() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0"));
		pls.add(1, ModelFactory.newPlayer("pl1"));
		
		myGame = ModelFactory.newGame(10, 10, pls);

		myGame.doMove(2, 1, myGame.getBoard().getPiece(0, 0));
		myGame.doMove(7, 8, myGame.getBoard().getPiece(9, 9));
		myGame.doMove(1, 2, myGame.getBoard().getPiece(0, 0));
		
		assertFalse(myGame.canStartCurrentMove(-1, -5));
		assertFalse(myGame.canStartCurrentMove(5, 5));
		assertFalse(myGame.canStartCurrentMove(0, 1));
		assertFalse(myGame.canStartCurrentMove(8, 7));
		
		assertFalse(myGame.canStartCurrentMove(0, 0));
		assertFalse(myGame.canStartCurrentMove(1, 2));
		assertFalse(myGame.canStartCurrentMove(2, 1));
		
		assertTrue(myGame.canStartCurrentMove(7, 8));
		assertTrue(myGame.canStartCurrentMove(9, 9));

		myGame.doMove(8, 7, myGame.getBoard().getPiece(9, 9));

		assertFalse(myGame.canStartCurrentMove(-1, -5));
		assertFalse(myGame.canStartCurrentMove(5, 5));
		assertFalse(myGame.canStartCurrentMove(0, 1));
		assertFalse(myGame.canStartCurrentMove(8, 7));

		assertFalse(myGame.canStartCurrentMove(0, 0));
		assertTrue(myGame.canStartCurrentMove(1, 2));
		assertTrue(myGame.canStartCurrentMove(2, 1));

		assertFalse(myGame.canStartCurrentMove(7, 8));
		assertFalse(myGame.canStartCurrentMove(9, 9));
		
		myGame.doMove(4, 2, myGame.getBoard().getPiece(2, 1));

		assertFalse(myGame.canStartCurrentMove(-1, -5));
		assertFalse(myGame.canStartCurrentMove(5, 5));
		assertFalse(myGame.canStartCurrentMove(0, 1));
		assertTrue(myGame.canStartCurrentMove(8, 7));

		assertFalse(myGame.canStartCurrentMove(0, 0));
		assertFalse(myGame.canStartCurrentMove(1, 2));
		assertFalse(myGame.canStartCurrentMove(2, 1));
		
		assertTrue(myGame.canStartCurrentMove(7, 8));
		assertFalse(myGame.canStartCurrentMove(9, 9));
		
		myGame.getBoard().getPiece(7, 8).kill();

		assertFalse(myGame.canStartCurrentMove(7, 8));
	}

	public void test11() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0"));
		pls.add(1, ModelFactory.newPlayer("pl1"));
		
		myGame = ModelFactory.newGame(10, 10, pls);

		myGame.doMove(2, 1, myGame.getBoard().getPiece(0, 0));
		myGame.doMove(7, 8, myGame.getBoard().getPiece(9, 9));
		myGame.doMove(1, 2, myGame.getBoard().getPiece(0, 0));
		
		assertFalse(myGame.canStartCurrentMove(-1, -5));
		assertFalse(myGame.canStartCurrentMove(5, 5));
		assertFalse(myGame.canStartCurrentMove(0, 1));
		assertFalse(myGame.canStartCurrentMove(8, 7));
		
		assertFalse(myGame.canStartCurrentMove(0, 0));
		assertFalse(myGame.canStartCurrentMove(1, 2));
		assertFalse(myGame.canStartCurrentMove(2, 1));
		
		assertTrue(myGame.canStartCurrentMove(7, 8));
		assertTrue(myGame.canStartCurrentMove(9, 9));

		myGame.doMove(8, 7, myGame.getBoard().getPiece(9, 9));

		assertFalse(myGame.canStartCurrentMove(-1, -5));
		assertFalse(myGame.canStartCurrentMove(5, 5));
		assertFalse(myGame.canStartCurrentMove(0, 1));
		assertFalse(myGame.canStartCurrentMove(8, 7));

		assertFalse(myGame.canStartCurrentMove(0, 0));
		assertTrue(myGame.canStartCurrentMove(1, 2));
		assertTrue(myGame.canStartCurrentMove(2, 1));

		assertFalse(myGame.canStartCurrentMove(7, 8));
		assertFalse(myGame.canStartCurrentMove(9, 9));
		
		myGame.doMove(4, 2, myGame.getBoard().getPiece(2, 1));

		assertFalse(myGame.canStartCurrentMove(-1, -5));
		assertFalse(myGame.canStartCurrentMove(5, 5));
		assertFalse(myGame.canStartCurrentMove(0, 1));
		assertTrue(myGame.canStartCurrentMove(8, 7));

		assertFalse(myGame.canStartCurrentMove(0, 0));
		assertFalse(myGame.canStartCurrentMove(1, 2));
		assertFalse(myGame.canStartCurrentMove(2, 1));
		
		assertTrue(myGame.canStartCurrentMove(7, 8));
		assertFalse(myGame.canStartCurrentMove(9, 9));
		
		assertTrue(myGame.canUndoMove());
		assertFalse(myGame.canRedoMove());
		
		myGame.undoMove();

		assertFalse(myGame.canStartCurrentMove(-1, -5));
		assertFalse(myGame.canStartCurrentMove(5, 5));
		assertFalse(myGame.canStartCurrentMove(0, 1));
		assertFalse(myGame.canStartCurrentMove(8, 7));

		assertFalse(myGame.canStartCurrentMove(0, 0));
		assertTrue(myGame.canStartCurrentMove(1, 2));
		assertTrue(myGame.canStartCurrentMove(2, 1));

		assertFalse(myGame.canStartCurrentMove(7, 8));
		assertFalse(myGame.canStartCurrentMove(9, 9));
		
		assertTrue(myGame.canUndoMove());
		assertTrue(myGame.canRedoMove());
		
		myGame.undoMove();
		
		assertFalse(myGame.canStartCurrentMove(-1, -5));
		assertFalse(myGame.canStartCurrentMove(5, 5));
		assertFalse(myGame.canStartCurrentMove(0, 1));
		assertFalse(myGame.canStartCurrentMove(8, 7));
		
		assertFalse(myGame.canStartCurrentMove(0, 0));
		assertFalse(myGame.canStartCurrentMove(1, 2));
		assertFalse(myGame.canStartCurrentMove(2, 1));
		
		assertTrue(myGame.canStartCurrentMove(7, 8));
		assertTrue(myGame.canStartCurrentMove(9, 9));

		assertTrue(myGame.canRedoMove());
		
		myGame.redoMove();
		
		assertFalse(myGame.canStartCurrentMove(-1, -5));
		assertFalse(myGame.canStartCurrentMove(5, 5));
		assertFalse(myGame.canStartCurrentMove(0, 1));
		assertFalse(myGame.canStartCurrentMove(8, 7));

		assertFalse(myGame.canStartCurrentMove(0, 0));
		assertTrue(myGame.canStartCurrentMove(1, 2));
		assertTrue(myGame.canStartCurrentMove(2, 1));

		assertFalse(myGame.canStartCurrentMove(7, 8));
		assertFalse(myGame.canStartCurrentMove(9, 9));

		assertTrue(myGame.canRedoMove());
		
		myGame.redoMove();
		
		assertFalse(myGame.canStartCurrentMove(-1, -5));
		assertFalse(myGame.canStartCurrentMove(5, 5));
		assertFalse(myGame.canStartCurrentMove(0, 1));
		assertTrue(myGame.canStartCurrentMove(8, 7));

		assertFalse(myGame.canStartCurrentMove(0, 0));
		assertFalse(myGame.canStartCurrentMove(1, 2));
		assertFalse(myGame.canStartCurrentMove(2, 1));
		
		assertTrue(myGame.canStartCurrentMove(7, 8));
		assertFalse(myGame.canStartCurrentMove(9, 9));
		
		assertFalse(myGame.canRedoMove());
		assertTrue(myGame.canUndoMove());
		
	}

	public void test12() {
		List<IPlayer> pls = new ArrayList<IPlayer>();
		
		pls.add(0, ModelFactory.newPlayer("pl0", false, "1"));
		pls.add(1, ModelFactory.newPlayer("pl1", false, "2"));
		
		myGame = ModelFactory.newGame(8, 8, pls);
		
		assertFalse(myGame.isGameOver());
		
		assertTrue(myGame.canDoMove(1, 2, myGame.getBoard().getPiece(0, 0), true));
		myGame.doMove(1, 2, myGame.getBoard().getPiece(0, 0));
		assertFalse(myGame.isGameOver());
		
		assertTrue(myGame.canDoMove(5, 6, myGame.getBoard().getPiece(7, 7), true));
		myGame.doMove(5, 6, myGame.getBoard().getPiece(7, 7));
		assertFalse(myGame.isGameOver());

		assertTrue(myGame.canDoMove(0, 4, myGame.getBoard().getPiece(1, 2), true));
		myGame.doMove(0, 4, myGame.getBoard().getPiece(1, 2));
		assertFalse(myGame.isGameOver());
		
		assertTrue(myGame.canDoMove(3, 5, myGame.getBoard().getPiece(5, 6), true));
		myGame.doMove(3, 5, myGame.getBoard().getPiece(5, 6));
		assertFalse(myGame.isGameOver());

		assertTrue(myGame.canDoMove(1, 6, myGame.getBoard().getPiece(0, 4), true));
		myGame.doMove(1, 6, myGame.getBoard().getPiece(0, 4));
		assertFalse(myGame.isGameOver());

		assertTrue(myGame.canDoMove(2, 3, myGame.getBoard().getPiece(3, 5), true));
		myGame.doMove(2, 3, myGame.getBoard().getPiece(3, 5));
		assertFalse(myGame.isGameOver());

		assertTrue(myGame.canDoMove(2, 5, myGame.getBoard().getPiece(0, 4), true));
		myGame.doMove(2, 5, myGame.getBoard().getPiece(0, 4));
		assertFalse(myGame.isGameOver());

		assertTrue(myGame.canDoMove(1, 1, myGame.getBoard().getPiece(2, 3), true));
		myGame.doMove(1, 1, myGame.getBoard().getPiece(2, 3));
		assertTrue(myGame.isGameOver());

		myGame.undoMove();
		assertFalse(myGame.isGameOver());

		myGame.undoMove();
		assertFalse(myGame.isGameOver());

		assertTrue(myGame.canDoMove(2, 1, myGame.getBoard().getPiece(0, 0), true));
		myGame.doMove(2, 1, myGame.getBoard().getPiece(0, 0));
		assertFalse(myGame.isGameOver());

		myGame.getBoard().getPiece(0, 4).kill();
		assertFalse(myGame.isGameOver());

		myGame.getBoard().getPiece(1, 6).kill();
		assertFalse(myGame.isGameOver());

		assertTrue(myGame.canDoMove(3, 1, myGame.getBoard().getPiece(2, 3), true));
		myGame.doMove(3, 1, myGame.getBoard().getPiece(2, 3));
		assertTrue(myGame.isGameOver());
	}
	
}
