package org.amse.vbut.vzab.view;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;

import org.amse.vbut.vzab.*;
import org.amse.vbut.vzab.model.*;

/**
 * VPanel class
 * 
 * @author But Vasiliy
 *
 */
@SuppressWarnings("serial")
public class VPanel extends JPanel {
	
	public static final int BORDER_SIZE = 2;
	public static final int PIECE_OUTER_SIZE = 3;
	
	private boolean myDragPiece;
	private IPiece myDraggedPiece;
	private int myDraggedX;
	private int myDraggedY;
	
	private IGame myGame;
	
	final private int myDefaultXSize;
	final private int myDefaultYSize;
	
	private boolean myDieMessageShow;
	private boolean myGrayMessageShow;
	private boolean myWinMessageShow;
	
	/*public boolean isModified() {
		IGame game = getGame();
		return (game == null) ? false : game.isModified();
	}*/
	
	public void setModified(boolean modified) {
		IGame game = getGame();
		if (game != null) {
			game.setModified(modified);
		}
	}
	
	public VPanel(int defaultxsize, int defaultysize) {
		super();
		
		myDefaultXSize = defaultxsize;
		myDefaultYSize = defaultysize;

		myDragPiece = false;
		myDraggedPiece = null;
		myDraggedX = 0;
		myDraggedY = 0;
		
		myGame = null;
		
		Dimension dim = new Dimension(defaultxsize * Main.CELL_SIZE + 2*BORDER_SIZE + 1,
				defaultysize * Main.CELL_SIZE + 2*BORDER_SIZE + 1);
		setPreferredSize(dim);
		setMaximumSize(dim);
		setMinimumSize(dim);
	}
	
	public void setGame(IGame game) {
		myGame = game;
		setModified(false);
		
		final IBoard b = game.getBoard();
		final int cellsize = Main.CELL_SIZE;
		
		Dimension dim = new Dimension(b.getWidth() * cellsize + 2*BORDER_SIZE + 1, 
				b.getHeight() * cellsize + 2*BORDER_SIZE + 1);

		setPreferredSize(dim);
		setMaximumSize(dim);
		setMinimumSize(dim);
	}
	
	public IGame getGame() {
		return myGame;
	}
	
	public void onMousePressed(MouseEvent me) {
		final IGame game = getGame();
		if (game != null && !isGameOver()) {
			final int cellx = getCellX(me.getX());
			final int celly = getCellY(me.getY());
			if (me.getButton() == MouseEvent.BUTTON1) {
				if (game.canStartCurrentMove(cellx, celly)) {
					myDragPiece = true;
					myDraggedX = cellx;
					myDraggedY = celly;
					myDraggedPiece = game.getBoard().getPiece(cellx, celly);
				}
			}
		}
	}

	public void onMouseReleased(MouseEvent me) {
		final IGame game = getGame();

		if (game != null) {
			final int cellx = getCellX(me.getX());
			final int celly = getCellY(me.getY());
			if (myDragPiece && myDraggedPiece != null) {
				
				if (game.canDoMove(cellx, celly, myDraggedPiece, true)) {
					game.doMove(cellx, celly, myDraggedPiece);
					
					final java.util.List<IPlayer> players = game.getPlayers(); 
					final int size = players.size();
					
					if (isGameOver()) {
						if (myWinMessageShow) {
							StringBuffer sb = new StringBuffer();
							int count = 0;
							
							for (int i = 0; i < size; ++i) {
								if (game.isPlayerAlive(i)) {
									if (++count > 1) {
										sb.append(", ");
									}
									sb.append(players.get(i).getName());
								}
							}
							
							String message = (count == 1 ? "Winner" : "Winners") 
								+ ": " + sb.toString() + ".";
							
							JOptionPane.showMessageDialog(VPanel.this, message, 
									"Game over", JOptionPane.INFORMATION_MESSAGE);
						}
					} else if (myDieMessageShow) {
						
						StringBuffer sb = new StringBuffer();
						int count = 0;
						
						for (int i = 0; i < size; ++i) {
							if (game.isPlayerKilled(i)) {

								if (++count > 1) {
									sb.append(", ");
								}
								
								sb.append(players.get(i).getName());
							}
						}
						
						if (count > 0) {
							String message;
							
							if (count > 1) {
								message = "Players " + sb.toString() + " have been killed.";
							} else {
								message = "Player " + sb.toString() + " has been killed.";
							}

							JOptionPane.showMessageDialog(VPanel.this, message, 
									"Player has been killed.", JOptionPane.INFORMATION_MESSAGE);
						}
					}

				} else {

					if (myGrayMessageShow && game.canDoMove(cellx, celly, myDraggedPiece, false)) {
						
						final IPlayer player = myDraggedPiece.getPlayer();
						final int playernum = game.getPlayerIndex(player);
						final BoardPoint bp = game.getPlayerBasePoint(playernum);
						final BoardRectangle br = game.getGrayedRectangle(bp);

						if ( br.inRect(cellx, celly) && game.isGrayedAreaFull(playernum)) {
								JOptionPane.showMessageDialog(VPanel.this,
										"Player \"" + player.getName() + "\" grayed area is filled.",
										"Incorrect move",
										JOptionPane.INFORMATION_MESSAGE);
							
						}
						
					}
				}
			}
		}
		myDragPiece = false;
		myDraggedPiece = null;
		myDraggedX = 0;
		myDraggedY = 0;
	}
	
	final private int getBoardWidth() {
		final IGame g = getGame();
		if (g == null) {
			return myDefaultXSize;
		} else {
			return g.getBoard().getWidth();
		}
	}

	final private int getBoardHeight() {
		final IGame g = getGame();
		if (g == null) {
			return myDefaultYSize;
		} else {
			return g.getBoard().getHeight();
		}
	}
	
	final private boolean isGameOver() {
		if (getGame() == null) {
			return true;
		}
		return getGame().isGameOver();
	}

	final private int getCellX(int mousex) {
		return (mousex - BORDER_SIZE) / Main.CELL_SIZE;
	}
	
	final private int getCellY(int mousey) {
		return (mousey - BORDER_SIZE) / Main.CELL_SIZE;
	}
	
	final private int getCellCoordinateX(int cellx) {
		return cellx * Main.CELL_SIZE + BORDER_SIZE;
	}

	final private int getCellCoordinateY(int celly) {
		return celly * Main.CELL_SIZE + BORDER_SIZE;
	}
	
	private Rectangle getGrayedRect(BoardPoint bp) {
		
		final IGame game = getGame();
		if (game == null) {
			return null;
		}

		BoardRectangle br = game.getGrayedRectangle(bp);
		
		final int rectanglex = getCellCoordinateX(br.getX0());
		final int rectangley = getCellCoordinateY(br.getY0());
		final int rectanglewidth = getCellCoordinateX(br.getX1() + 1) - rectanglex;
		final int rectangleheight = getCellCoordinateY(br.getY1() + 1) - rectangley;
		
		return new Rectangle(rectanglex, rectangley, rectanglewidth, rectangleheight);
	}

	private void drawGrid(Graphics g)
	{
		final int xsize = getBoardWidth();
		final int ysize = getBoardHeight();
		
		IGame game = getGame();
		if (game != null) {
			final java.util.List<IPlayer> players = game.getPlayers();
			final int size = players.size();
			
			for (int i = 0; i < size; ++i) {
				BoardPoint bp = game.getPlayerBasePoint(i);
				
				if (bp == null) {
					continue;
				}
				
				Rectangle rect = getGrayedRect(bp);
				
				g.setColor( Color.LIGHT_GRAY );
				
				g.fillRect(rect.x, rect.y, rect.width, rect.height);
				
				g.setColor( Color.BLACK );
				
				g.drawRect(rect.x + 1, rect.y + 1, rect.width - 2, rect.height - 2);
			}
			
		}
		
		g.setColor(Color.BLACK);
		
		for (int i = 0; i <= xsize; ++i) {
			g.drawLine(getCellCoordinateX(i), 
					getCellCoordinateY(0), 
					getCellCoordinateX(i), 
					getCellCoordinateY(ysize));
		}

		for (int j = 0; j <= ysize; ++j) {
			g.drawLine(getCellCoordinateX(0), 
					getCellCoordinateY(j), 
					getCellCoordinateX(xsize), 
					getCellCoordinateY(j));
		}
	}
	
	/*private void drawPiece(Graphics g, int x, int y, int border, Color color) {
		g.setColor(color);
		
		final int cellsize = getCellSize();
		g.fillOval(x + border, y + border, cellsize - 2*border, cellsize - 2*border);
	}*/
	
	final private ImageIcon receiveImage(int cellx, int celly, IPiece piece) {
		final IGame game = getGame();
		final IPlayer player = piece.getPlayer();
		final BoardPoint bp = game.getPlayerBasePoint(game.getPlayerIndex(player));
		final boolean alive = piece.isAlive();
		
		if (bp.getX() == cellx && bp.getY() == celly) {
			if (alive) {
				return player.getImageBase();
			} else {
				return player.getImageBaseKilled();
			}
		} else {
			if (alive) {
				return player.getImageAlive();
			} else {
				return player.getImageKilled();
			}
		}
	}
	
	private void drawPiece(Graphics g, int piececellx, int piececelly, IPiece piece, 
			int outer) {
		
		drawPiece(g, getCellCoordinateX(piececellx), getCellCoordinateY(piececelly),
				receiveImage(piececellx, piececelly, piece), 
				piececellx, piececelly, outer);
		
	}
	
	private void drawPiece(Graphics g, int x, int y, ImageIcon image, 
			int piececellx, int piececelly, int outer) {
		
		final int cellsize = Main.CELL_SIZE;
		
		final int imagewidth = image.getIconWidth();
		final int imageheight = image.getIconHeight();
		
		final int dx = (cellsize - imagewidth) / 2;
		final int dy = (cellsize - imageheight) / 2;

		g.drawImage(image.getImage(), x + dx - outer, y + dy - outer, 
				x + cellsize - dx + outer, y + cellsize - dy + outer, 
				0, 0, imagewidth, imageheight, this);
		
	}
	
	private void drawCell(Graphics g, int cellx, int celly, boolean iscurrent, IPiece piece) {
		if (iscurrent && getGame().canStartCurrentMove(cellx, celly) && 
				!myDragPiece && !isGameOver()) {
			
			drawPiece(g, cellx,	celly, piece, PIECE_OUTER_SIZE);
		} else {
			drawPiece(g, cellx,	celly, piece, 0);
		}
	}
	
	private void drawAllPieces(Graphics g, int currentcellx, int currentcelly) {
		final IGame game = getGame();
		if (game != null) {
			final int xsize = getBoardWidth();
			final int ysize = getBoardHeight();
			for (int j = 0; j < ysize; ++j) {
				for (int i = 0; i < xsize; ++i) {
					final IPiece piece = game.getBoard().getPiece(i, j);
					if (piece != null) {
						final boolean iscurrent = currentcellx == i && currentcelly == j; 
						drawCell(g, i, j, iscurrent, piece);
					}
				}
			}
		}
	}
	
	public void paint(Graphics g) {
		super.paint(g);
		drawGrid(g);
		
		if (getGame() != null) {
			final Point mouseposition = getMousePosition();
			
			final int currentcellx = (mouseposition != null) ? 
					getCellX(mouseposition.x) : -1;
					
			final int currentcelly = (mouseposition != null) ? 
					getCellY(mouseposition.y) : -1;
			
			drawAllPieces(g, currentcellx, currentcelly);
			
			if (myDragPiece && mouseposition != null) {
				int piecepositionx;
				int piecepositiony;
				if (getGame().canDoMove(currentcellx, currentcelly, myDraggedPiece, true)) {
					piecepositionx = getCellCoordinateX(currentcellx);
					piecepositiony = getCellCoordinateY(currentcelly);
				} else {
					final int cellsize = Main.CELL_SIZE;
					piecepositionx = mouseposition.x - cellsize / 2;
					piecepositiony = mouseposition.y - cellsize / 2;
				}
				
				drawPiece(g, piecepositionx, piecepositiony, 
						myDraggedPiece.getPlayer().getImageAlive(), 
						myDraggedX, myDraggedY, 0); 
			}
		} else {
			Font font = g.getFont();
			g.setFont( new Font(font.getName(), Font.BOLD, 26) );
			g.setColor(Color.RED);
			
			String message = "Press \"New Game\" button";
			FontMetrics fm = g.getFontMetrics();
			Rectangle2D rect = fm.getStringBounds(message, g);
			int x = (getWidth() - (int)rect.getWidth()) / 2;
			int y = getHeight() / 2 - (int)rect.getHeight();
			g.drawString(message, x, y);
			
			message = "to start the game.";
			rect = fm.getStringBounds(message, g);
			x = (getWidth() - (int)rect.getWidth()) / 2;
			y = getHeight() / 2;
			g.drawString(message, x, y);
		}
	}

	public boolean isWinMessageShow() {
		return myWinMessageShow;
	}
	
	public boolean isDieMessageShow() {
		return myDieMessageShow;
	}

	public boolean isGrayMessageShow() {
		return myGrayMessageShow;
	}

	public void setWinMessageShow(boolean show) {
		myWinMessageShow = show;
	}
	
	public void setDieMessageShow(boolean show) {
		myDieMessageShow = show;
	}

	public void setGrayMessageShow(boolean show) {
		myGrayMessageShow = show;
	}

}

