package org.amse.vbut.vzab.view.dialogs;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.amse.vbut.vzab.model.*;
import org.amse.vbut.vzab.view.tools.*; 

/**
 * EditPlayerDialog class
 * 
 * @author But Vasiliy
 *
 */
@SuppressWarnings("serial")
public class EditPlayerDialog extends JDialog {
	
	private IPlayerShell myPlayer;
	
	private JTextField myNameField;
	//private JCheckBox myIsComputerCheck;
	
	private JButton myOkButton, myCancelButton;
	private boolean myResult;

	public EditPlayerDialog(Dialog owner, IPlayerShell player) {
		super(owner, "edit player data", true);
		
		myPlayer = player;
		
		myResult = false;
		
		getContentPane().add(createMainPanel());
		
		myNameField.setText(player.getName());
		//myIsComputerCheck.setSelected(player.isComputer());

		pack();
		setResizable(false);
		
		setLocation( owner.getX() + (owner.getWidth() - getWidth()) / 2, 
				owner.getY() + (owner.getHeight() - getHeight()) / 2);
		
		setVisible(true);
	}
	
	private JPanel createNamePanel() {
		JPanel panel = BoxLayoutUtils.createHorizontalPanel();
		JLabel label = new JLabel("Name:");
		myNameField = new JTextField();
		panel.add(label);
		panel.add(Box.createHorizontalStrut(5));
		panel.add(myNameField);
		return panel;
	}
	
	private JPanel createConfirmPanel() {
		JPanel panelFlow = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
		JPanel panelGrid = new JPanel(new GridLayout(1, 2, 5, 0));
		
		myOkButton = new JButton(new OkAction());
		myCancelButton = new JButton(new CancelAction());
		
		panelGrid.add(myOkButton);
		panelGrid.add(myCancelButton);
		panelFlow.add(panelGrid);
		
		panelFlow.setMinimumSize(panelFlow.getPreferredSize());
		
		Dimension dim = panelFlow.getMaximumSize();
		panelFlow.setMaximumSize(new Dimension(dim.width, panelFlow.getPreferredSize().height));

		BoxLayoutUtils.setGroupAlignmentY(new JComponent[] {myOkButton, 
				myCancelButton}, Component.CENTER_ALIGNMENT);
		GUITools.createRecommendedMargin(new JButton[] {myOkButton, myCancelButton});
		
		getRootPane().setDefaultButton(myOkButton);
		
		return panelFlow;
	}

	private JPanel createMainPanel() {
		JPanel panelMain = BoxLayoutUtils.createVerticalPanel();
		panelMain.setBorder(BorderFactory.createEmptyBorder(12, 12, 12, 12));
		
		JPanel panelName = createNamePanel();
		
		//myIsComputerCheck = new JCheckBox("Is computer");

		JPanel panelConfirm = createConfirmPanel();
	
		BoxLayoutUtils.setGroupAlignmentX(
				new JComponent[] {panelName, panelConfirm, 
						/*myIsComputerCheck*/},	Component.LEFT_ALIGNMENT);
		
		GUITools.createRecommendedMargin(new JButton[] {myOkButton, myCancelButton});
		GUITools.fixComponentHeight(myNameField);
		
		panelMain.add(panelName);
		panelMain.add(Box.createVerticalStrut(12));
		//panelMain.add(myIsComputerCheck);
		panelMain.add(Box.createVerticalStrut(17));
		panelMain.add(panelConfirm);
		
		return panelMain;
	}
	
	private class OkAction extends AbstractAction {
		public OkAction() {
			super("Ok");
		}
		
		public void actionPerformed(ActionEvent ae) {
			myPlayer.setName(myNameField.getText());
			//myPlayer.setComputer(myIsComputerCheck.isSelected());
			myResult = true;
			dispose();
		}
	}
	
	private class CancelAction extends AbstractAction {
		public CancelAction() {
			super("Cancel");
		}
		
		public void actionPerformed(ActionEvent ae) {
			myResult = false;
			dispose();
		}
	}
	
	public boolean getResult() {
		return myResult;
	}
}
