package org.amse.vbut.vzab.view.dialogs;

import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import org.amse.vbut.vzab.model.*;
import org.amse.vbut.vzab.model.impl.*;
import org.amse.vbut.vzab.view.Main;
import org.amse.vbut.vzab.view.tools.*;

/**
 * NewGameDialog class
 * 
 * @author But Vasiliy
 *
 */
@SuppressWarnings("serial")
public class NewGameDialog extends JDialog {

	private IGame myGame;
	
	private SpinnerNumberModel myWidthSpinnerModel;
	private SpinnerNumberModel myHeightSpinnerModel;
	private SpinnerNumberModel myPlayerSpinnerModel;
	
	private JPanel myBoxesPanel;
	
	private JLabel myStatusLabel;
	
	private JComboBox[] myPlayerBoxes;
	
	private OkAction myOkAction;
	
	private boolean myResult;
	
	private int mySpinnerMax;
	
	public IGame getGame() {
		return myGame;
	}
	
	public boolean getResult() {
		return myResult;
	}
	
	public NewGameDialog(Frame owner, IGame oldgame, java.util.List<IPlayer> players) {
		super(owner, "Create new game", true);
		
		myResult = false;
		myPlayerBoxes = new JComboBox[IGame.MAX_PLAYERS];
		
		final ItemListener l = new BoxItemListener();
		final Object[] playersarray = players.toArray();
		
		final PlayerListCellRenderer renderer = new PlayerListCellRenderer(); 
		
		for (int i = 0; i < IGame.MAX_PLAYERS; ++i) {
			final JComboBox box = new JComboBox(playersarray);
			GUITools.fixComponentHeight(box);
			box.addItemListener(l);
			box.setToolTipText("Player number " + (i + 1));
			box.setRenderer(renderer);
			myPlayerBoxes[i] = box;
		}
		
		getContentPane().add(createMainPanel(oldgame, players));
		
		pack();
		setResizable(false);
		
		setLocation( owner.getX() + (owner.getWidth() - getWidth()) / 2, 
				owner.getY() + (owner.getHeight() - getHeight()) / 2);
		
		setVisible(true);
	}
	
	private JPanel createWidthHeightPanel() {
		JPanel panel = BoxLayoutUtils.createVerticalPanel();
		
		JPanel panelWidth = BoxLayoutUtils.createHorizontalPanel();
		JPanel panelHeight = BoxLayoutUtils.createHorizontalPanel();

		myWidthSpinnerModel = new SpinnerNumberModel(IGame.MIN_WIDTH, IGame.MIN_WIDTH, Main.getMaxWidth(), 1);
		myHeightSpinnerModel = new SpinnerNumberModel(IGame.MIN_HEIGHT, IGame.MIN_HEIGHT, Main.getMaxHeight(), 1);

		JLabel labelw = new JLabel("Width:");
		JLabel labelh = new JLabel("Height:");
		GUITools.makeSameSize(new JComponent[] {labelw, labelh});

		JSpinner spinnerWidth = new JSpinner(myWidthSpinnerModel);
		JSpinner spinnerHeight = new JSpinner(myHeightSpinnerModel);

		spinnerWidth.setToolTipText("Width of the board: integer from " + IGame.MIN_WIDTH + " to " + Main.getMaxWidth());
		spinnerHeight.setToolTipText("Heigth of the board: integer from " + IGame.MIN_HEIGHT + " to " + Main.getMaxHeight());

		GUITools.fixComponentHeight(spinnerWidth);
		GUITools.fixComponentHeight(spinnerHeight);

		panelWidth.add(labelw);
		panelWidth.add(Box.createHorizontalStrut(5));
		panelWidth.add(spinnerWidth);
		
		panelHeight.add(labelh);
		panelHeight.add(Box.createHorizontalStrut(5));
		panelHeight.add(spinnerHeight);

		panel.add(panelWidth);
		panel.add(Box.createVerticalStrut(5));
		panel.add(panelHeight);

		return panel;
	}
	
	private JPanel createSpinPanel(IGame oldgame, java.util.List<IPlayer> players) {
		JPanel panel = BoxLayoutUtils.createHorizontalPanel();
		JLabel label = new JLabel("Number of players:");

		final int spinnervalue = (oldgame == null) ? IGame.MIN_PLAYERS : oldgame.getPlayers().size();
		mySpinnerMax = (players.size() < IGame.MAX_PLAYERS) ? players.size() : IGame.MAX_PLAYERS;
		
		myPlayerSpinnerModel = new SpinnerNumberModel(spinnervalue, IGame.MIN_PLAYERS, mySpinnerMax, 1); 
		myPlayerSpinnerModel.addChangeListener(new PlayerSpinnerListener());
		JSpinner playerspinner = new JSpinner(myPlayerSpinnerModel);
		playerspinner.setToolTipText("Number of players: integer from " + IGame.MIN_PLAYERS + " to " + mySpinnerMax);

		GUITools.fixComponentHeight(playerspinner);

		panel.add(label);
		panel.add(Box.createHorizontalStrut(5));
		panel.add(playerspinner);
		return panel;
	}
	
	private JPanel createLeftPanel(IGame oldgame, java.util.List<IPlayer> players) {
		JPanel panel = BoxLayoutUtils.createVerticalPanel();
		
		JPanel panelWidthHeight = createWidthHeightPanel();
		JPanel panelSpin = createSpinPanel(oldgame, players);
		
		BoxLayoutUtils.setGroupAlignmentX(new JComponent[] {panelWidthHeight,
				panelSpin}, Component.LEFT_ALIGNMENT);

		panel.add(panelWidthHeight);
		panel.add(Box.createVerticalStrut(12));
		panel.add(panelSpin);
		panel.add(Box.createVerticalGlue());
		
		GUITools.fixComponentHeight(panel);
		
		return panel;
	}
	
	private JPanel createWorkPanel(IGame oldgame, java.util.List<IPlayer> players) {
		JPanel panel = BoxLayoutUtils.createHorizontalPanel();
		
		JPanel panelLeft = createLeftPanel(oldgame, players); 
			
		myBoxesPanel = BoxLayoutUtils.createVerticalPanel();
		
		for (int i = 0; i < myPlayerBoxes.length; ++i) {
			if (i > 0) {
				myBoxesPanel.add(Box.createVerticalStrut(5));
			}
			myBoxesPanel.add(myPlayerBoxes[i]);
		}
		myBoxesPanel.add(Box.createVerticalGlue());
		
		panel.add(panelLeft);
		panel.add(Box.createHorizontalStrut(12));
		panel.add(myBoxesPanel);
		
		BoxLayoutUtils.setGroupAlignmentY(new JComponent[] {panelLeft,
				myBoxesPanel}, JComponent.TOP_ALIGNMENT);
		
		return panel;
	}
	
	private JLabel createStatusLabel() {
		final JLabel label = new JLabel(" ", SwingConstants.RIGHT);
		
		label.setForeground(java.awt.Color.RED);
		
		Font font = label.getFont();
		
		label.setFont(new Font(font.getName(), Font.ITALIC, font.getSize()));
		
		return label;
	}
	
	private JPanel createMainPanel(IGame oldgame, java.util.List<IPlayer> players) {
		JPanel panelMain = BoxLayoutUtils.createVerticalPanel();
		panelMain.setBorder(BorderFactory.createEmptyBorder(12, 12, 12, 12));
		
		myStatusLabel = createStatusLabel();
		
		JPanel panelWork = createWorkPanel(oldgame, players);
		
		JPanel panelConfirm = createConfirmPanel();
		
		panelMain.add(panelWork);
		panelMain.add(Box.createVerticalStrut(5));
		panelMain.add(myStatusLabel);
		panelMain.add(Box.createVerticalStrut(5));
		panelMain.add(panelConfirm);

		BoxLayoutUtils.setGroupAlignmentX( new JComponent[] {panelWork, 
				panelConfirm, myStatusLabel}, JLabel.RIGHT_ALIGNMENT);
		
		if (oldgame != null) {
			final IBoard board = oldgame.getBoard();
			myWidthSpinnerModel.setValue( new Integer(board.getWidth()) );
			myHeightSpinnerModel.setValue( new Integer(board.getHeight()) );
		} else {
			myWidthSpinnerModel.setValue( new Integer(IGame.DEFAULT_WIDTH) );
			myHeightSpinnerModel.setValue( new Integer(IGame.DEFAULT_HEIGHT) );
		}

		composePlayers(mySpinnerMax, oldgame);
		updateOkButton();
		final int number = myPlayerSpinnerModel.getNumber().intValue(); 
		updateBoxes(number);
		
		return panelMain;
	}
	
	private JPanel createConfirmPanel() {
		JPanel panelFlow = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
		JPanel panelGrid = new JPanel(new GridLayout(1, 2, 5, 0));

		myOkAction = new OkAction();
		myOkAction.setEnabled(false);
		
		JButton buttonOk = new JButton(myOkAction);
		JButton buttonCancel = new JButton(new CancelAction());
		
		panelGrid.add(buttonOk);
		panelGrid.add(buttonCancel);
		panelFlow.add(panelGrid);
		
		panelFlow.setMinimumSize(panelFlow.getPreferredSize());
		
		Dimension dim = panelFlow.getMaximumSize();
		panelFlow.setMaximumSize(new Dimension(dim.width, panelFlow.getPreferredSize().height));

		BoxLayoutUtils.setGroupAlignmentY(new JComponent[] {buttonOk, 
				buttonCancel}, Component.CENTER_ALIGNMENT);
		GUITools.createRecommendedMargin(new JButton[] {buttonOk, buttonCancel});
		
		getRootPane().setDefaultButton(buttonOk);

		return panelFlow;
	}

	
	private class OkAction extends AbstractAction {
		public OkAction() {
			super("Ok");
		}
		
		public void actionPerformed(ActionEvent arg0) {
			
			final int width = myWidthSpinnerModel.getNumber().intValue();
			final int height = myHeightSpinnerModel.getNumber().intValue();
			
			java.util.List<IPlayer> players = new java.util.LinkedList<IPlayer>();
			final int number = myPlayerSpinnerModel.getNumber().intValue();
			for (int i = 0; i < number; ++i) {
				players.add((IPlayer)myPlayerBoxes[i].getSelectedItem());
			}

			myGame = ModelFactory.newGame(width, height, players);
			myResult = true;
			dispose();
		}
		
		public boolean isEnabled() {
			final int number = myPlayerSpinnerModel.getNumber().intValue();
			
			for (int i = 1; i < number; ++i) {
				for (int j = 0; j < i; ++j) {
					if (myPlayerBoxes[i].getSelectedIndex() ==
						myPlayerBoxes[j].getSelectedIndex()) {
						
						myStatusLabel.setText("Error: player " + j + " and player " + i + " are the same.");
						
						return false;
					}
				}
			}
			
			myStatusLabel.setText(" ");

			return true;
		}
	}

	private class CancelAction extends AbstractAction {
		public CancelAction() {
			super("Cancel");
		}
		
		public void actionPerformed(ActionEvent arg0) {
			myGame = null;
			myResult = false;
			dispose();
		}
	}
	
	private void updateBoxes(int number) {
		for (int i = 0; i < number; ++i) {
			myPlayerBoxes[i].setEnabled(true);
		}
		for (int i = number; i < myPlayerBoxes.length; ++i) {
			myPlayerBoxes[i].setEnabled(false);
		}
	}
	
	private void composePlayers(int maxplayers, IGame oldgame) {
		if (oldgame != null) {
			final java.util.List<IPlayer> players = oldgame.getPlayers();
			final int playernum = players.size();
			
			for (int i = 0; i < playernum; ++i) {
				myPlayerBoxes[i].setSelectedItem(players.get(i));
			}
			
			for (int i = playernum; i < maxplayers; ++i) {
				int index = i;
				for (int j = 0; j < playernum; ++j) {
					boolean flag = true;
					for (int k = 0; k < i; ++k) {
						if (myPlayerBoxes[k].getSelectedIndex() == j) {
							flag = false;
							break;
						}
					}
					if (flag) {
						index = j;
						break;
					}
				}
				myPlayerBoxes[i].setSelectedIndex(index);
			}
		} else {
			for (int i = 0; i < maxplayers; ++i) {
				myPlayerBoxes[i].setSelectedIndex(i);
			}
		}
	}
	
	private void updateOkButton() {
		//System.out.println(myOkAction.isEnabled());
		myOkAction.setEnabled(myOkAction.isEnabled());
	}

	private class PlayerSpinnerListener implements ChangeListener {
		public void stateChanged(ChangeEvent ce) {
			updateBoxes(myPlayerSpinnerModel.getNumber().intValue());
			updateOkButton();
		}
	}
	
	private class BoxItemListener implements ItemListener {
		public void itemStateChanged(ItemEvent ie) {
			updateOkButton();
		}
	}
	
}
