package org.amse.vbut.vzab.view.events;

import javax.swing.*;

import java.awt.event.*;

import org.amse.vbut.vzab.model.*;
import org.amse.vbut.vzab.view.*;
import org.amse.vbut.vzab.view.dialogs.NewGameDialog;
import org.amse.vbut.vzab.io.*;

@SuppressWarnings("serial")
public class NewGameAction extends AbstractAction {

	private Viewer myView;
	
	public NewGameAction(Viewer view) {
		super("new game", IOTools.loadIcon(Main.IMAGES_PATH + "/new.png"));
		putValue(AbstractAction.SHORT_DESCRIPTION, "Start new game");
		myView = view;
	}

	public void actionPerformed(ActionEvent ae) {
		final VPanel panel = myView.getVPanel();
		final IGame game = panel.getGame();

		if (game != null && game.isModified()) {

			int result = JOptionPane.showConfirmDialog(myView, 
					"Current game is modified.\nDo you want to save it?", "Save game", 
					JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
			
			switch (result) {
			case JOptionPane.YES_OPTION: 
				if (!SaveGameAction.saveGame(myView)) {
					return;
				}
				break;
				
			case JOptionPane.NO_OPTION:
				break;
				
			default: 
				return;
					
			}
			
		}
		
		NewGameDialog dlg = new NewGameDialog(myView, game, myView.getPlayers());
		
		if (dlg.getResult()) {
			panel.setGame(dlg.getGame());
		}
		
		myView.updateDimensions();
		myView.updateView();
	}
	
	public boolean isEnabled() {
		
		if (myView.getPlayers().size() < IGame.MIN_PLAYERS) {
			return false;
		}
		
		return true;
	}

}
