package org.amse.vbut.vzab.view.events;

import javax.swing.*;

import java.awt.event.ActionEvent;

import org.amse.vbut.vzab.io.*;
import org.amse.vbut.vzab.io.impl.*;
import org.amse.vbut.vzab.*;
import org.amse.vbut.vzab.view.*;
import org.amse.vbut.vzab.model.*;

@SuppressWarnings("serial")
public class OpenGameAction extends AbstractAction {

	private Viewer myView;
	
	public OpenGameAction(Viewer view) {
		super("open game", IOTools.loadIcon(Main.IMAGES_PATH + "/open.png"));
		putValue(AbstractAction.SHORT_DESCRIPTION, "Open saved game");
		myView = view;
	}
	
	public static boolean openGame(Viewer view) {
		
		JFileChooser  chooser = new JFileChooser(".");
		chooser.setMultiSelectionEnabled(false);
		chooser.setDialogTitle("Open saved game...");
		chooser.addChoosableFileFilter( new ExtensionFileFilter( Main.GAME_EXTENSION, 
				"VZab game file (*." + Main.GAME_EXTENSION + ")" ) );

		int result = chooser.showOpenDialog(view);
		
		if (result == JFileChooser.APPROVE_OPTION) {
			try {
				IReader reader = IOFactory.newReader();
				
				final IGame game = reader.readGame(chooser.getSelectedFile().getAbsolutePath(),
						IGame.MIN_WIDTH, IGame.MIN_HEIGHT, Main.getMaxWidth(), Main.getMaxHeight());
				
				view.getVPanel().setGame(game);
				
				final java.util.List<IPlayer> players = view.getPlayers();
				final java.util.List<IPlayer> gameplayers = game.getPlayers();
				final int size = gameplayers.size();
				
				for (int i = 0; i < size; ++i) {
					IPlayer p = gameplayers.get(i);
					if (!players.contains(p)) {
						players.add(p);
					}
				}
				
			} catch (VZabException e) {
				JOptionPane.showMessageDialog(view, 
						"Unable to open file:\n\"" 
						+ chooser.getSelectedFile().getAbsolutePath() 
						+ "\".\nError: " + e.getMessage(),
						"Error",
						JOptionPane.ERROR_MESSAGE);
				return false;
			}
			view.updateDimensions();
			view.updateView();
			
			return true;
		}
		
		return false;
	}
	
	public void actionPerformed(ActionEvent arg0) {
		final IGame game = myView.getVPanel().getGame();
		
		if (game != null && game.isModified()) {
			int result = JOptionPane.showConfirmDialog(myView, 
					"Current game is modified.\nDo you want to save it?", "Save game", 
					JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
			
			switch (result) {
			case JOptionPane.YES_OPTION: 
				if (!SaveGameAction.saveGame(myView)) {
					return;
				}
				break;
				
			case JOptionPane.NO_OPTION:
				break;
				
			default: 
				return;
					
			}
		}

		openGame(myView);
	}
	
}
